/*
 * Kuklomenos
 * Copyright (C) 2008 Martin Bays <mbays@sdf.lonestar.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 */

#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>
#include <sstream>
using namespace std;

#include "conffile.h"
#include "SDL_gfxPrimitivesDirty.h"

int powmod(int n, int m, int p)
{
    int out = 1;
    while (m > 0)
	if (m % 2 == 0)
	{
	    n = (n*n) % p;
	    m /= 2;
	}
	else
	{
	    out = (out*n) % p;
	    m -= 1;
	}
    return out;

}

/* obfuscate the rating string: if you want to cheat, you at least have to
 * source-dive...
 */
int obfuscatedRating(double rating)
{
    int pertinentRating = (int)(10*rating);
    return powmod(pertinentRating, 5, 45953);
}
double unobfuscatedRating(int obfuscated)
{
    double unobf = powmod(obfuscated, 18381, 45953)/10.0;
    if (unobf < 0 || unobf > 20)
	return 0.0;
    return unobf;
}

Config::Config() :
    rating(5.0),
    useAA(AA_YES), showGrid(true), zoomEnabled(true),
    rotatingView(true), fps(30), showFPS(true), music(false),
    aaGamma(2.2),
    shouldUpdateRating(false) {}

void Config::read()
{
    ifstream f;
    char* home = getenv("HOME");

    if (home != NULL)
    {
	string homeStr = home;
	f.open((home + (string)"/.kuklomenosrc").c_str());
    }
    if (!f.is_open())
	f.open("./kuklomenosrc.txt");

    if (f.is_open())
    {
	string line;
	double val;

	while (!f.eof())
	{
	    std::getline(f, line);
	    const char* cstr = line.c_str();
	    if (sscanf(cstr, "data: %lf", &val) == 1)
		rating = unobfuscatedRating( (int)(val) );
	    else if (sscanf(cstr, "showGrid: %lf", &val) == 1)
		showGrid = val;
	    else if (sscanf(cstr, "zoomEnabled: %lf", &val) == 1)
		zoomEnabled = val;
	    else if (sscanf(cstr, "rotatingView: %lf", &val) == 1)
		rotatingView = val;
	    else if (sscanf(cstr, "showFPS: %lf", &val) == 1)
		showFPS = val;
	    else if (sscanf(cstr, "music: %lf", &val) == 1)
		music = val;
	    else if (sscanf(cstr, "antialiasGamma: %lf", &val) == 1)
		aaGamma = val;
	    else if (sscanf(cstr, "fps: %lf", &val) == 1)
		fps = (int) val;
	    else if (sscanf(cstr, "useAA: %lf", &val) == 1)
		useAA = (val == 0 ? AA_NO :
			val == 2 ? AA_FORCE :
			AA_YES);
	    else if (*cstr != '\0' && *cstr != '#')
		fprintf(stderr, "Unparsable line in config file: %s\n", cstr);
	}
    }
}

void Config::importSettings(const Settings& settings)
{
    useAA = settings.useAA;
    showGrid = settings.showGrid;
    zoomEnabled = settings.zoomEnabled;
    rotatingView = settings.rotatingView;
    fps = settings.fps;
    showFPS = settings.showFPS;
    music = settings.music;

    aaGamma = antialiasGamma;
}
void Config::exportSettings(Settings& settings) const
{
    settings.useAA = useAA;
    settings.showGrid = showGrid;
    settings.zoomEnabled = zoomEnabled;
    settings.rotatingView = rotatingView;
    settings.fps = fps;
    settings.showFPS = showFPS;
    settings.music = music;

    antialiasGamma = aaGamma;
}

void Config::write() const
{
    ofstream f;
    char* home = getenv("HOME");

    if (home != NULL)
    {
	string homeStr = home;
	f.open((home + (string)"/.kuklomenosrc").c_str());
    }
    if (!f.is_open())
	f.open("./kuklomenosrc.txt");

    if (!f.is_open())
	fprintf(stderr, "Couldn't write config file.\n");
    else
    {
	ostringstream s;
	int aanum = (useAA == AA_NO ? 0 :
		useAA == AA_YES ? 1 : 2);
	s <<
	    "useAA: " << aanum << endl <<
	    "showGrid: " << showGrid << endl <<
	    "zoomEnabled: " << zoomEnabled << endl <<
	    "rotatingView: " << rotatingView << endl <<
	    "fps: " << fps << endl <<
	    "showFPS: " << showFPS << endl <<
	    "music: " << music << endl <<
	    "antialiasGamma: " << aaGamma << endl <<
	     endl << "# Warning: Do not edit following line" << endl <<
	    "data: " << obfuscatedRating(rating) << endl; 
	f << s.str();
    }
}

Config config;
