/*
 * Kuklomenos
 * Copyright (C) 2008 Martin Bays <mbays@sdf.lonestar.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 */

#include <string>

#include "menu.h"
#include "settings.h"
#include "SDL_gfxPrimitivesDirty.h"

#include <config.h>

#ifdef HAVE_LIBSDL_MIXER
#define MUSIC 1
#endif

Menu::Menu(std::string title,
	Menu* m1, MenuLeaf* ml1,
	Menu* m2, MenuLeaf* ml2,
	Menu* m3, MenuLeaf* ml3,
	Menu* m4, MenuLeaf* ml4) :
    title(title)
{
    menus[0] = m1;
    menus[1] = m2;
    menus[2] = m3;
    menus[3] = m4;

    leaves[0] = ml1;
    leaves[1] = ml2;
    leaves[2] = ml3;
    leaves[3] = ml4;
}

std::string MenuLeafToggleBool::name()
{
    return varName +
	((*var) ?
	 ": true" :
	 ": false");
}

LeafReturn MenuLeafToggleBool::act()
{
    *var = !*var;
    return LR_NONE;
}

std::string MenuLeafCycleAA::name()
{
    switch (settings.useAA)
    {
	case AA_NO: return "antialias: none";
	case AA_YES: return "antialias: some";
	case AA_FORCE: default: return "antialias: all";
    }
}

LeafReturn MenuLeafCycleAA::act()
{
    settings.useAA =
	(settings.useAA == AA_NO) ? AA_YES :
	(settings.useAA == AA_YES) ? AA_FORCE :
	AA_NO;
    return LR_NONE;
}

MenuLeafToggleBool zoomToggle("Zoom", &settings.zoomEnabled);
MenuLeafToggleBool rotToggle("Rotate", &settings.rotatingView);
MenuLeafToggleBool gridToggle("Grid", &settings.showGrid);
Menu viewMenu("View",
	NULL, &zoomToggle,
	NULL, &rotToggle,
	NULL, &gridToggle
	);

#ifdef MUSIC
MenuLeafToggleBool musicToggle("Music", &settings.music);
#endif

MenuLeafCycleAA cycleAALeaf;
MenuLeafIncVar<double> decAlphaLeaf(&antialiasGamma, "AA Gamma", -0.1, 0.1);
MenuLeafIncVar<double> incAlphaLeaf(&antialiasGamma, "AA Gamma", 0.1, 0.1);
MenuLeafShowVar<double> showAlphaLeaf(&antialiasGamma, "AA Gamma");
Menu AAMenu("Anti-Aliasing",
	NULL, &decAlphaLeaf,
	NULL, &cycleAALeaf,
	NULL, &incAlphaLeaf,
	NULL, &showAlphaLeaf);

MenuLeafToggleBool showFPSToggle("Show FPS", &settings.showFPS);
MenuLeafIncVar<int> decFPSLeaf(&settings.fps, "FPS", -1, 2);
MenuLeafIncVar<int> incFPSLeaf(&settings.fps, "FPS", 1, 2);
MenuLeafShowVar<int> showFPSLeaf(&settings.fps, "FPS");
Menu fpsMenu("Framerate",
	NULL, &decFPSLeaf,
	NULL, &showFPSToggle,
	NULL, &incFPSLeaf,
	NULL, &showFPSLeaf
	);

Menu graphicsMenu("Graphics",
	NULL, NULL,
	&fpsMenu, NULL,
	NULL, NULL,
	&AAMenu, NULL
	);

MenuLeafReturn saveLeaf("Save settings", LR_SAVESETTINGS);
Menu settingsMenu("Settings",
	&graphicsMenu, NULL,
#ifdef MUSIC
	NULL, &musicToggle,
#else
	NULL, NULL,
#endif
	&viewMenu, NULL,
	NULL, &saveLeaf
	);

MenuLeafReturn quitLeaf("Quit", LR_QUIT);
MenuLeafReturn surrenderLeaf("Surrender", LR_SURRENDER);
Menu quitMenu("Quit",
	NULL, &quitLeaf,
	NULL, NULL,
	NULL, &surrenderLeaf
	);

MenuLeafReturn resumeLeaf("Resume", LR_EXITMENU);

Menu topMenu("top",
	NULL, &resumeLeaf,
	NULL, NULL,
	&settingsMenu, NULL,
	&quitMenu, NULL
	);
