#ifndef INC_MENU_H
#define INC_MENU_H

#include <string>
#include <iostream>
#include <sstream>

enum MenuNodeType
{
    MNT_NONE,
    MNT_MENU,
    MNT_LEAF
};
enum LeafReturn
{
    LR_NONE,
    LR_QUIT,
    LR_SURRENDER,
    LR_EXITMENU,
    LR_SAVESETTINGS
};

class Menu;
class MenuLeaf;

class Menu
{
    public:
	std::string title;

	/* menus and leaves give the menu or leaf to be found in each
	 * direction; 0->west, 1->north, 2->east, 3->south. For each i, at
	 * most one of menus[i] and leaves[i] should be non-NULL.
	 */
	Menu* menus[4];
	MenuLeaf* leaves[4];

	Menu();

	Menu(std::string title,
		Menu* m1=NULL, MenuLeaf* ml1=NULL,
		Menu* m2=NULL, MenuLeaf* ml2=NULL,
		Menu* m3=NULL, MenuLeaf* ml3=NULL,
		Menu* m4=NULL, MenuLeaf* ml4=NULL
		);
};

class MenuLeaf
{
    public:
	virtual std::string name() =0;
	virtual LeafReturn act() =0;

	virtual ~MenuLeaf() {};
};

class MenuLeafToggleBool : public MenuLeaf
{
    private:
	std::string varName;
	bool* var;
    public:
	std::string name();
	LeafReturn act();

	MenuLeafToggleBool(std::string varName, bool* var) :
	    varName(varName), var(var) {}
};

class MenuLeafCycleAA : public MenuLeaf
{
    public:
	std::string name();
	LeafReturn act();

	MenuLeafCycleAA() {}
};

template<typename T>
class MenuLeafIncVar : public MenuLeaf
{
    private:
	T* var;
	std::string varName;
	T amount;
	T min;
    public:
	std::string name() {
	    return ( (amount >= 0) ? "Inc " : "Dec " ) + varName;
	}
	LeafReturn act() {
	    *var += amount;
	    if (*var < min)
		*var = min;
	    return LR_NONE;
	}

	MenuLeafIncVar(T* var, std::string(varName), T amount, T min = 0) :
	    var(var), varName(varName), amount(amount), min(min) {}
};

template<typename T>
class MenuLeafShowVar : public MenuLeaf
{
    private:
	T* var;
	std::string varName;
    public:
	std::string name() {
	    std::stringstream ss;
	    ss << varName << ": " << *var;
	    return ss.str();
	}
	LeafReturn act() { return LR_NONE; }

	MenuLeafShowVar(T* var, std::string(varName)) :
	    var(var), varName(varName) {}
};

class MenuLeafReturn : public MenuLeaf
{
    private:
	std::string text;
	LeafReturn ret;
    public:
	std::string name() { return text; }
	LeafReturn act() { return ret; }

	MenuLeafReturn(std::string text, LeafReturn ret) :
	    text(text), ret(ret) {}
};

extern Menu topMenu;

#endif /* INC_MENU_H */
